@@ -5,7 +5,7 @@
 //
 
 /*
-Copyright or © or Copr. CNRS, (November 17, 2004)
+Copyright or © or Copr. Bio++ Development Team, (November 17, 2004)
 
 This software is a computer program whose purpose is to provide classes
 for numerical calculus.
@@ -532,9 +532,10 @@ class InfinityFunctionWrapper:
         constraintMatch_ = false;
         return test;
       }
-      catch(ConstraintException& ce)
+      catch (ConstraintException& ce)
       {
         constraintMatch_ = true;
+        return false;
       }
     }
 
