@@ -5,7 +5,7 @@
 //
 
 /*
-   Copyright or © or Copr. CNRS, (November 17, 2004)
+   Copyright or © or Copr. Bio++ Development Team, (November 17, 2004)
 
    This software is a computer program whose purpose is to provide classes
    for numerical calculus. This file is part of the Bio++ project.
@@ -210,7 +210,7 @@ public:
     unsigned int nrB = B.getNumberOfRows();
     unsigned int ncB = B.getNumberOfColumns();
     if (ncA != nrB) throw DimensionException("MatrixTools::mult(). nrows B != ncols A.", nrB, ncA);
-    if (ncA != D.size()) throw DimensionException("MatrixTools::mult(). Vector size is not eual to matrix size.", D.size(), ncA);
+    if (ncA != D.size()) throw DimensionException("MatrixTools::mult(). Vector size is not equal to matrix size.", D.size(), ncA);
     O.resize(nrA, ncB);
     for (unsigned int i = 0; i < nrA; i++)
     {
@@ -225,6 +225,52 @@ public:
     }
   }
 
+  /**
+   * @brief Compute A . (U+D+L) . B where D is a diagonal matrix, U
+   * (resp. L) is a matrix in which the only non-zero terms are on the
+   * diagonal that is over (resp. under) the main diagonal, in O(n^3).
+   *
+   * Since D is a diagonal matrix, this function is more efficient than doing
+   * mult(mult(A, diag(D)), B), which involves two 0(n^3) operations.
+   *
+   * @param A [in] The first matrix.
+   * @param D [in] The diagonal matrix (only diagonal elements in a vector)
+   * @param U [in] The upper diagonal matrix (only upper diagonal elements in a vector)
+   * @param L [in] The lower diagonal matrix (only lower diagonal elements in a vector)
+   * @param B [in] The second matrix.
+   * @param O [out] The result matrix.
+   * @throw DimensionException If matrices have not the appropriate size.
+   */
+  template<class Scalar>
+  static void mult(const Matrix<Scalar>& A, const std::vector<Scalar>& D, const std::vector<Scalar>& U, const std::vector<Scalar>& L, const Matrix<Scalar>& B, Matrix<Scalar>& O) throw (DimensionException)
+  {
+    unsigned int ncA = A.getNumberOfColumns();
+    unsigned int nrA = A.getNumberOfRows();
+    unsigned int nrB = B.getNumberOfRows();
+    unsigned int ncB = B.getNumberOfColumns();
+    if (ncA != nrB) throw DimensionException("MatrixTools::mult(). nrows B != ncols A.", nrB, ncA);
+    if (ncA != D.size()) throw DimensionException("MatrixTools::mult(). Vector size is not equal to matrix size.", D.size(), ncA);
+    if (ncA != U.size()+1) throw DimensionException("MatrixTools::mult(). Vector size is not equal to matrix size-1.", U.size(), ncA);
+    if (ncA != L.size()+1) throw DimensionException("MatrixTools::mult(). Vector size is not equal to matrix size-1.", L.size(), ncA);
+    O.resize(nrA, ncB);
+    for (unsigned int i = 0; i < nrA; i++)
+      {
+        for (unsigned int j = 0; j < ncB; j++)
+          {
+            O(i, j) = A(i, 0) * D[0] * B(0, j);
+            if (nrB>1)
+              O(i, j) += A(i,0) * U[0] * B(1,j);
+            for (unsigned int k = 1; k < ncA-1; k++)
+              {
+                O(i, j) += A(i, k) * (L[k-1] * B(k-1, j) + D[k] * B(k, j) + U[k] * B(k+1,j));
+              }
+            if (ncA>=2)
+              O(i, j) += A(i, ncA-1) * L[ncA-2] * B(ncA-2, j);
+            O(i,j) += A(i, ncA-1) * D[ncA-1] * B(ncA-1, j);
+          }
+      }
+  }
+
   /**
    * @brief Add matrix B to matrix A.
    *
@@ -791,6 +837,288 @@ public:
     }
     return sum;
   }
+
+
+  
+  /**
+   * @brief Linear Assignment Problem
+   *
+   * The algorithm coded here is described in 
+   * * A Shortest Augmenting Path Algorithm for Dense and Sparse Linear Assignment Problems, Computing 38, 325-340, 1987
+   * by R. Jonker and A. Volgenant, University of Amsterdam.
+   *
+   * @param assignCost [input/output] Cost matrix
+   * @param rowSol     [output] Column assigned to row in solution
+   * @param colSol     [output] Row assigned to column in solution
+   * @param u          [output] Dual variables, row reduction numbers
+   * @param v          [output] Dual variables, column reduction numbers
+   * @return The optimal cost.
+   */
+  template<class Scalar>
+  static Scalar lap(Matrix<Scalar>& assignCost,
+      std::vector<int> &rowSol, 
+      std::vector<int> &colSol, 
+      std::vector<Scalar> &u, 
+      std::vector<Scalar> &v) throw (Exception)
+  {
+    unsigned int dim = assignCost.getNumberOfRows();
+    if (assignCost.getNumberOfColumns() != dim)
+      throw Exception("MatrixTools::lap. Cost matrix should be scare.");
+  
+    bool unassignedFound;
+    unsigned int i, iMin;
+    unsigned int numFree = 0, previousNumFree, f, i0, k, freeRow;
+    std::vector<unsigned int> free(dim); // list of unassigned rows.
+    std::vector<unsigned int> pred(dim); // row-predecessor of column in augmenting/alternating path.
+    unsigned int j, j1, j2, endOfPath, last, low, up;
+    std::vector<unsigned int> colList(dim); // list of columns to be scanned in various ways.
+    std::vector<short int> matches(dim, 0); // counts how many times a row could be assigned.
+    Scalar min;
+    Scalar h;
+    unsigned int uMin, uSubMin;
+    Scalar v2;
+    std::vector<Scalar> d(dim); // 'cost-distance' in augmenting path calculation.
+
+    // Column reduction
+    for (j = dim; j > 0; j--)    // reverse order gives better results.
+    {
+      // find minimum cost over rows.
+      min = assignCost(0, j - 1); 
+      iMin = 0;
+      for (i = 1; i < dim; ++i) { 
+        if (assignCost(i, j - 1) < min) 
+        { 
+          min = assignCost(i, j - 1); 
+          iMin = i;
+        }
+      }
+      v[j - 1] = min; 
+
+      if (++matches[iMin] == 1) 
+      { 
+        // init assignment if minimum row assigned for first time.
+        rowSol[iMin] = j - 1; 
+        colSol[j - 1] = iMin; 
+      }
+      else
+        colSol[j - 1] = -1;        // row already assigned, column not assigned.
+    }
+
+    // Reduction tranfer
+    for (i = 0; i < dim; i++) { 
+      if (matches[i] == 0)     // fill list of unassigned 'free' rows.
+        free[numFree++] = i;
+      else {
+        if (matches[i] == 1)   // transfer reduction from rows that are assigned once.
+        {
+          j1 = rowSol[i]; 
+          min = -log(0);
+          for (j = 0; j < dim; j++)  
+            if (j != j1)
+              if (assignCost(i, j - 1) - v[j] < min) 
+                min = assignCost(i, j - 1) - v[j - 1];
+          v[j1] = v[j1] - min;
+        }
+      }
+    }
+
+    // Augmenting row reduction 
+    short loopcnt = 0;           // do-loop to be done twice.
+    do
+    {
+      loopcnt++;
+
+      // scan all free rows.
+      // in some cases, a free row may be replaced with another one to be scanned next.
+      k = 0; 
+      previousNumFree = numFree; 
+      numFree = 0;             // start list of rows still free after augmenting row reduction.
+      while (k < previousNumFree)
+      {
+        i = free[k]; 
+        k++;
+
+        // find minimum and second minimum reduced cost over columns.
+        uMin = assignCost(i, 0) - v[0]; 
+        j1 = 0; 
+        uSubMin = -log(0);
+        for (j = 1; j < dim; j++) 
+        {
+          h = assignCost(i, j) - v[j];
+          if (h < uSubMin) {
+            if (h >= uMin) 
+            { 
+              uSubMin = h; 
+              j2 = j;
+            }
+            else 
+            { 
+              uSubMin = uMin; 
+              uMin = h; 
+              j2 = j1; 
+              j1 = j;
+            }
+          }
+        }
+
+        i0 = colSol[j1];
+        if (uMin < uSubMin) {
+          // change the reduction of the minimum column to increase the minimum
+          // reduced cost in the row to the subminimum.
+          v[j1] = v[j1] - (uSubMin - uMin);
+        } else {                  // minimum and subminimum equal.
+          if (i0 >= 0)         // minimum column j1 is assigned.
+          { 
+            // swap columns j1 and j2, as j2 may be unassigned.
+            j1 = j2; 
+            i0 = colSol[j2];
+          }
+        }
+
+        // (re-)assign i to j1, possibly de-assigning an i0.
+        rowSol[i] = j1; 
+        colSol[j1] = i;
+
+        if (i0 >= 0) {          // minimum column j1 assigned earlier.
+          if (uMin < uSubMin) {
+            // put in current k, and go back to that k.
+            // continue augmenting path i - j1 with i0.
+            free[--k] = i0; 
+          } else { 
+            // no further augmenting reduction possible.
+            // store i0 in list of free rows for next phase.
+            free[numFree++] = i0; 
+          }
+        }
+      }
+    }
+    while (loopcnt < 2);       // repeat once.
+
+    // Augment solution for each free row.
+    for (f = 0; f < numFree; f++) 
+    {
+      freeRow = free[f];       // start row of augmenting path.
+
+      // Dijkstra shortest path algorithm.
+      // runs until unassigned column added to shortest path tree.
+      for (j = 0; j < dim; j++)  
+      { 
+        d[j] = assignCost(freeRow, j) - v[j]; 
+        pred[j] = freeRow;
+        colList[j] = j;        // init column list.
+      }
+
+      low = 0; // columns in 0..low-1 are ready, now none.
+      up = 0;  // columns in low..up-1 are to be scanned for current minimum, now none.
+               // columns in up..dim-1 are to be considered later to find new minimum, 
+               // at this stage the list simply contains all columns 
+      unassignedFound = false;
+      do
+      {
+        if (up == low)         // no more columns to be scanned for current minimum.
+        {
+          last = low - 1; 
+
+          // scan columns for up..dim-1 to find all indices for which new minimum occurs.
+          // store these indices between low..up-1 (increasing up). 
+          min = d[colList[up++]]; 
+          for (k = up; k < dim; k++) 
+          {
+            j = colList[k]; 
+            h = d[j];
+            if (h <= min)
+            {
+              if (h < min)     // new minimum.
+              { 
+                up = low;      // restart list at index low.
+                min = h;
+              }
+              // new index with same minimum, put on undex up, and extend list.
+              colList[k] = colList[up]; 
+              colList[up++] = j; 
+            }
+          }
+
+          // check if any of the minimum columns happens to be unassigned.
+          // if so, we have an augmenting path right away.
+          for (k = low; k < up; k++) { 
+            if (colSol[colList[k]] < 0) 
+            {
+              endOfPath = colList[k];
+              unassignedFound = true;
+              break;
+            }
+          }
+        }
+
+        if (!unassignedFound) 
+        {
+          // update 'distances' between freerow and all unscanned columns, via next scanned column.
+          j1 = colList[low]; 
+          low++; 
+          i = colSol[j1]; 
+          h = assignCost(i, j1) - v[j1] - min;
+
+          for (k = up; k < dim; k++) 
+          {
+            j = colList[k]; 
+            v2 = assignCost(i, j) - v[j] - h;
+            if (v2 < d[j])
+            {
+              pred[j] = i;
+              if (v2 == min) {  // new column found at same minimum value
+                if (colSol[j] < 0) 
+                {
+                  // if unassigned, shortest augmenting path is complete.
+                  endOfPath = j;
+                  unassignedFound = true;
+                  break;
+                }
+                // else add to list to be scanned right away.
+                else 
+                { 
+                  colList[k] = colList[up]; 
+                  colList[up++] = j; 
+                }
+              }
+              d[j] = v2;
+            }
+          }
+        } 
+      }
+      while (!unassignedFound);
+
+      // update column prices.
+      for (k = 0; k <= last; k++)  
+      { 
+        j1 = colList[k]; 
+        v[j1] = v[j1] + d[j1] - min;
+      }
+
+      // reset row and column assignments along the alternating path.
+      do
+      {
+        i = pred[endOfPath]; 
+        colSol[endOfPath] = i; 
+        j1 = endOfPath; 
+        endOfPath = rowSol[i]; 
+        rowSol[i] = j1;
+      }
+      while (i != freeRow);
+    }
+
+    // calculate optimal cost.
+    Scalar lapCost = 0;
+    for (i = 0; i < dim; i++)  
+    {
+      j = rowSol[i];
+      u[i] = assignCost(i, j) - v[j];
+      lapCost = lapCost + assignCost(i, j); 
+    }
+
+    return lapCost;
+  }
+
 };
 
 /* DEPRECATED
